@@ -319,7 +319,8 @@ class TestNovaServerBasic(base.SenlinTestCase):
             'source_type': 'image',
             'device_type': None,
             'destination_type': 'volume',
-            'delete_on_termination': None
+            'delete_on_termination': None,
+            'volume_type': None
         }]
         mock_volume.assert_called_once_with(
             node_obj, expected_volume, 'create')
@@ -370,7 +371,60 @@ class TestNovaServerBasic(base.SenlinTestCase):
             'source_type': 'volume',
             'device_type': None,
             'destination_type': 'volume',
-            'delete_on_termination': None
+            'delete_on_termination': None,
+            'volume_type': None
+        }]
+        mock_volume.assert_called_once_with(
+            node_obj, expected_volume, 'create')
+
+    def test_do_create_bdm_invalid_volume_type(self):
+        cc = mock.Mock()
+        nc = mock.Mock()
+        node_obj = mock.Mock(id='FAKE_NODE_ID', data={}, index=123,
+                             cluster_id='FAKE_CLUSTER_ID')
+        bdm_v2 = [
+            {
+                'volume_size': 1,
+                'source_type': 'blank',
+                'destination_type': 'volume',
+                'boot_index': 0,
+                'volume_type': '588854a9',
+            },
+        ]
+        spec = {
+            'type': 'os.nova.server',
+            'version': '1.0',
+            'properties': {
+                'flavor': 'FLAV',
+                'name': 'FAKE_SERVER_NAME',
+                'security_groups': ['HIGH_SECURITY_GROUP'],
+                'block_device_mapping_v2': bdm_v2,
+            }
+        }
+        profile = server.ServerProfile('s2', spec)
+        profile._computeclient = cc
+        profile._networkclient = nc
+        self._stubout_profile(profile, mock_image=True, mock_flavor=True,
+                              mock_keypair=True)
+        err = exc.EResourceCreation(type='server', message='FOO')
+        mock_volume = self.patchobject(profile, '_resolve_bdm',
+                                       side_effect=err)
+
+        self.assertRaises(exc.EResourceCreation,
+                          profile.do_create,
+                          node_obj)
+        expected_volume = [{
+            'guest_format': None,
+            'boot_index': 0,
+            'uuid': None,
+            'volume_size': 1,
+            'device_name': None,
+            'disk_bus': None,
+            'source_type': 'blank',
+            'device_type': None,
+            'destination_type': 'volume',
+            'delete_on_termination': None,
+            'volume_type': '588854a9'
         }]
         mock_volume.assert_called_once_with(
             node_obj, expected_volume, 'create')
@@ -437,7 +491,8 @@ class TestNovaServerBasic(base.SenlinTestCase):
             'fixed_ip': None,
             'floating_ip': None,
             'port': None,
-            'security_groups': None
+            'security_groups': None,
+            'subnet': None
         }
         mock_net.assert_called_once_with(
             node_obj, expect_params, 'create')
@@ -623,7 +678,8 @@ class TestNovaServerBasic(base.SenlinTestCase):
             'source_type': 'image',
             'device_type': None,
             'destination_type': 'volume',
-            'delete_on_termination': None
+            'delete_on_termination': None,
+            'volume_type': None
         }
         self.assertEqual(expected_volume,
                          profile.properties['block_device_mapping_v2'][0])
