@@ -11,11 +11,11 @@
 # under the License.
 
 import copy
+from unittest import mock
 
-import mock
 from oslo_context import context as oslo_ctx
-import six
 
+from senlin.common import consts
 from senlin.common import context as senlin_ctx
 from senlin.common import exception
 from senlin.common import schema
@@ -84,6 +84,7 @@ class TestProfileBase(base.SenlinTestCase):
         self.ctx = utils.dummy_context(project='profile_test_project')
         g_env = environment.global_env()
         g_env.register_profile('os.dummy-1.0', DummyProfile)
+        g_env.register_profile('os.dummy-1.1', DummyProfile)
         self.spec = parser.simple_parse(sample_profile)
 
     def _create_profile(self, name, pid=None, context=None):
@@ -130,6 +131,72 @@ class TestProfileBase(base.SenlinTestCase):
         self.assertIsNone(profile._orchestrationclient)
         self.assertIsNone(profile._block_storageclient)
 
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
+    def test_init_version_as_float(self, mock_creds):
+        mock_creds.return_value = {'foo': 'bar'}
+        self.spec['version'] = 1.1
+        profile = self._create_profile('test-profile')
+
+        self.assertIsNone(profile.id)
+        self.assertEqual('test-profile', profile.name)
+        self.assertEqual(self.spec, profile.spec)
+        self.assertEqual('os.dummy', profile.type_name)
+        self.assertEqual('1.1', profile.version)
+        self.assertEqual('os.dummy-1.1', profile.type)
+        self.assertEqual(self.ctx.user_id, profile.user)
+        self.assertEqual(self.ctx.project_id, profile.project)
+        self.assertEqual(self.ctx.domain_id, profile.domain)
+        self.assertEqual({}, profile.metadata)
+        self.assertIsNone(profile.created_at)
+        self.assertIsNone(profile.updated_at)
+
+        spec_data = profile.spec_data
+        self.assertEqual('os.dummy', spec_data['type'])
+        self.assertEqual('1.1', spec_data['version'])
+        self.assertEqual('value1', spec_data['properties']['key1'])
+        self.assertEqual(2, spec_data['properties']['key2'])
+        self.assertEqual('value1', profile.properties['key1'])
+        self.assertEqual(2, profile.properties['key2'])
+        self.assertEqual({'foo': 'bar'}, profile.context)
+
+        self.assertIsNone(profile._computeclient)
+        self.assertIsNone(profile._networkclient)
+        self.assertIsNone(profile._orchestrationclient)
+        self.assertIsNone(profile._block_storageclient)
+
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
+    def test_init_version_as_string(self, mock_creds):
+        mock_creds.return_value = {'foo': 'bar'}
+        self.spec['version'] = '1.1'
+        profile = self._create_profile('test-profile')
+
+        self.assertIsNone(profile.id)
+        self.assertEqual('test-profile', profile.name)
+        self.assertEqual(self.spec, profile.spec)
+        self.assertEqual('os.dummy', profile.type_name)
+        self.assertEqual('1.1', profile.version)
+        self.assertEqual('os.dummy-1.1', profile.type)
+        self.assertEqual(self.ctx.user_id, profile.user)
+        self.assertEqual(self.ctx.project_id, profile.project)
+        self.assertEqual(self.ctx.domain_id, profile.domain)
+        self.assertEqual({}, profile.metadata)
+        self.assertIsNone(profile.created_at)
+        self.assertIsNone(profile.updated_at)
+
+        spec_data = profile.spec_data
+        self.assertEqual('os.dummy', spec_data['type'])
+        self.assertEqual('1.1', spec_data['version'])
+        self.assertEqual('value1', spec_data['properties']['key1'])
+        self.assertEqual(2, spec_data['properties']['key2'])
+        self.assertEqual('value1', profile.properties['key1'])
+        self.assertEqual(2, profile.properties['key2'])
+        self.assertEqual({'foo': 'bar'}, profile.context)
+
+        self.assertIsNone(profile._computeclient)
+        self.assertIsNone(profile._networkclient)
+        self.assertIsNone(profile._orchestrationclient)
+        self.assertIsNone(profile._block_storageclient)
+
     @mock.patch.object(senlin_ctx, 'get_service_credentials')
     def test_init_with_context(self, mock_creds):
         mock_creds.return_value = {'foo': 'bar'}
@@ -155,7 +222,7 @@ class TestProfileBase(base.SenlinTestCase):
         ex = self.assertRaises(exception.ESchema,
                                pb.Profile, 'test-profile', bad_spec)
         self.assertEqual("The 'version' key is missing from the provided "
-                         "spec map.", six.text_type(ex))
+                         "spec map.", str(ex))
 
     def test_from_object(self):
         obj = self._create_profile('test_profile_for_record')
@@ -234,7 +301,7 @@ class TestProfileBase(base.SenlinTestCase):
 
         self.assertEqual("Failed in creating profile my_profile: The "
                          "profile_type 'bogus-1.0' could not be found.",
-                         six.text_type(ex))
+                         str(ex))
 
     @mock.patch.object(pb.Profile, 'validate')
     @mock.patch.object(senlin_ctx, 'get_service_credentials')
@@ -247,7 +314,7 @@ class TestProfileBase(base.SenlinTestCase):
                                self.ctx, 'my_profile', self.spec)
 
         self.assertEqual("Failed in creating profile my_profile: "
-                         "Boom", six.text_type(ex))
+                         "Boom", str(ex))
 
     @mock.patch.object(po.Profile, 'delete')
     def test_delete(self, mock_delete):
@@ -374,10 +441,10 @@ class TestProfileBase(base.SenlinTestCase):
             side_effect=exception.InternalError(code=400, message='BAD'))
         obj = mock_load
 
-        res = profile.check_object(self.ctx, obj)
+        self.assertRaises(exception.InternalError, profile.check_object,
+                          self.ctx, obj)
 
         profile.load(self.ctx).do_check.assert_called_once_with(obj)
-        self.assertFalse(res)
 
     @mock.patch.object(pb.Profile, 'load')
     def test_update_object_with_profile(self, mock_load):
@@ -570,7 +637,7 @@ class TestProfileBase(base.SenlinTestCase):
         self.assertRaises(exception.ESchema, profile.validate)
 
     @mock.patch.object(senlin_ctx, 'get_service_credentials')
-    def test__init_context(self, mock_creds):
+    def test_init_context(self, mock_creds):
         fake_ctx = mock.Mock()
         mock_creds.return_value = fake_ctx
 
@@ -585,7 +652,7 @@ class TestProfileBase(base.SenlinTestCase):
         mock_creds.assert_called_once_with()
 
     @mock.patch.object(senlin_ctx, 'get_service_credentials')
-    def test__init_context_for_real(self, mock_creds):
+    def test_init_context_for_real(self, mock_creds):
         fake_ctx = {
             'project_name': 'this project',
             'project_domain_name': 'this domain',
@@ -607,7 +674,7 @@ class TestProfileBase(base.SenlinTestCase):
         self.assertEqual(expected, profile.context)
 
     @mock.patch.object(senlin_ctx, 'get_service_credentials')
-    def test__init_context_for_real_with_data(self, mock_creds):
+    def test_init_context_for_real_with_data(self, mock_creds):
         fake_ctx = {
             'project_name': 'this project',
             'project_domain_name': 'this domain',
@@ -633,7 +700,7 @@ class TestProfileBase(base.SenlinTestCase):
 
     @mock.patch.object(co.Credential, 'get')
     @mock.patch.object(oslo_ctx, 'get_current')
-    def test__build_conn_params(self, mock_current, mock_get):
+    def test_build_conn_params(self, mock_current, mock_get):
         profile = self._create_profile('test-profile')
         profile.context = {'foo': 'bar'}
         fake_cred = mock.Mock(cred={'openstack': {'trust': 'TRUST_ID'}})
@@ -655,7 +722,7 @@ class TestProfileBase(base.SenlinTestCase):
 
     @mock.patch.object(co.Credential, 'get')
     @mock.patch.object(oslo_ctx, 'get_current')
-    def test__build_conn_params_trust_not_found(self, mock_current, mock_get):
+    def test_build_conn_params_trust_not_found(self, mock_current, mock_get):
         profile = self._create_profile('test-profile')
         mock_get.return_value = None
         fake_ctx = mock.Mock()
@@ -696,6 +763,25 @@ class TestProfileBase(base.SenlinTestCase):
 
         self.assertEqual(cc, res)
 
+    @mock.patch.object(pb.Profile, '_build_conn_params')
+    @mock.patch("senlin.drivers.base.SenlinDriver")
+    def test_glance_client(self, mock_senlindriver, mock_params):
+        obj = mock.Mock()
+        sd = mock.Mock()
+        gc = mock.Mock()
+        sd.glance.return_value = gc
+        mock_senlindriver.return_value = sd
+        fake_params = mock.Mock()
+        mock_params.return_value = fake_params
+        profile = self._create_profile('test-profile')
+
+        res = profile.glance(obj)
+
+        self.assertEqual(gc, res)
+        self.assertEqual(gc, profile._glanceclient)
+        mock_params.assert_called_once_with(obj.user, obj.project)
+        sd.glance.assert_called_once_with(fake_params)
+
     @mock.patch.object(pb.Profile, '_build_conn_params')
     @mock.patch("senlin.drivers.base.SenlinDriver")
     def test_neutron_client(self, mock_senlindriver, mock_params):
@@ -751,11 +837,12 @@ class TestProfileBase(base.SenlinTestCase):
         self.patchobject(profile, 'do_create', return_value=True)
         self.patchobject(profile, 'do_delete', return_value=True)
 
-        res, status = profile.do_recover(mock.Mock())
+        res, status = profile.do_recover(mock.Mock(),
+                                         operation=consts.RECOVER_RECREATE)
         self.assertTrue(status)
 
         res, status = profile.do_recover(
-            mock.Mock(), operation=[{'name': 'bar'}])
+            mock.Mock(), operation='bar')
         self.assertFalse(status)
 
     def test_do_recover_with_fencing(self):
@@ -765,12 +852,12 @@ class TestProfileBase(base.SenlinTestCase):
         obj = mock.Mock()
 
         res = profile.do_recover(obj, ignore_missing=True,
-                                 params={"fence_compute": True})
+                                 params={"fence_compute": True},
+                                 operation=consts.RECOVER_RECREATE)
 
         self.assertTrue(res)
-        profile.do_delete.assert_called_once_with(obj, force=True,
-                                                  timeout=None,
-                                                  delete_ports_on_failure=None)
+        profile.do_delete.assert_called_once_with(obj, force=False,
+                                                  timeout=None)
         profile.do_create.assert_called_once_with(obj)
 
     def test_do_recover_with_delete_timeout(self):
@@ -779,12 +866,12 @@ class TestProfileBase(base.SenlinTestCase):
         self.patchobject(profile, 'do_delete', return_value=True)
         obj = mock.Mock()
 
-        res = profile.do_recover(obj, ignore_missing=True, delete_timeout=5)
+        res = profile.do_recover(obj, ignore_missing=True, delete_timeout=5,
+                                 operation=consts.RECOVER_RECREATE)
 
         self.assertTrue(res)
         profile.do_delete.assert_called_once_with(obj, force=False,
-                                                  timeout=5,
-                                                  delete_ports_on_failure=None)
+                                                  timeout=5)
         profile.do_create.assert_called_once_with(obj)
 
     def test_do_recover_with_force_recreate(self):
@@ -793,12 +880,12 @@ class TestProfileBase(base.SenlinTestCase):
         self.patchobject(profile, 'do_delete', return_value=True)
         obj = mock.Mock()
 
-        res = profile.do_recover(obj, ignore_missing=True, force_recreate=True)
+        res = profile.do_recover(obj, ignore_missing=True, force_recreate=True,
+                                 operation=consts.RECOVER_RECREATE)
 
         self.assertTrue(res)
         profile.do_delete.assert_called_once_with(obj, force=False,
-                                                  timeout=None,
-                                                  delete_ports_on_failure=True)
+                                                  timeout=None)
         profile.do_create.assert_called_once_with(obj)
 
     def test_do_recover_with_force_recreate_failed_delete(self):
@@ -809,11 +896,11 @@ class TestProfileBase(base.SenlinTestCase):
         self.patchobject(profile, 'do_delete', side_effect=err)
         obj = mock.Mock()
 
-        res = profile.do_recover(obj, ignore_missing=True, force_recreate=True)
+        res = profile.do_recover(obj, ignore_missing=True, force_recreate=True,
+                                 operation=consts.RECOVER_RECREATE)
         self.assertTrue(res)
         profile.do_delete.assert_called_once_with(obj, force=False,
-                                                  timeout=None,
-                                                  delete_ports_on_failure=True)
+                                                  timeout=None)
         profile.do_create.assert_called_once_with(obj)
 
     def test_do_recover_with_false_force_recreate_failed_delete(self):
@@ -821,7 +908,7 @@ class TestProfileBase(base.SenlinTestCase):
         err = exception.EResourceDeletion(type='STACK', id='ID',
                                           message='BANG')
         self.patchobject(profile, 'do_delete', side_effect=err)
-        operation = [{"name": "RECREATE"}]
+        operation = "RECREATE"
 
         ex = self.assertRaises(exception.EResourceOperation,
                                profile.do_recover,
@@ -829,13 +916,13 @@ class TestProfileBase(base.SenlinTestCase):
                                force_recreate=False)
         self.assertEqual("Failed in recovering node 'NODE_ID': "
                          "Failed in deleting STACK 'ID': BANG.",
-                         six.text_type(ex))
+                         str(ex))
 
     def test_do_recover_with_recreate_succeeded(self):
         profile = self._create_profile('test-profile')
         self.patchobject(profile, 'do_delete', return_value=True)
         self.patchobject(profile, 'do_create', return_value=True)
-        operation = [{"name": "RECREATE"}]
+        operation = "RECREATE"
         res = profile.do_recover(mock.Mock(), operation=operation)
 
         self.assertTrue(res)
@@ -845,21 +932,21 @@ class TestProfileBase(base.SenlinTestCase):
         err = exception.EResourceDeletion(type='STACK', id='ID',
                                           message='BANG')
         self.patchobject(profile, 'do_delete', side_effect=err)
-        operation = [{"name": "RECREATE"}]
+        operation = "RECREATE"
 
         ex = self.assertRaises(exception.EResourceOperation,
                                profile.do_recover,
                                mock.Mock(id='NODE_ID'), operation=operation)
         self.assertEqual("Failed in recovering node 'NODE_ID': "
                          "Failed in deleting STACK 'ID': BANG.",
-                         six.text_type(ex))
+                         str(ex))
 
     def test_do_recover_with_recreate_failed_create(self):
         profile = self._create_profile('test-profile')
         self.patchobject(profile, 'do_delete', return_value=True)
         err = exception.EResourceCreation(type='STACK', message='BANNG')
         self.patchobject(profile, 'do_create', side_effect=err)
-        operation = [{"name": "RECREATE"}]
+        operation = "RECREATE"
 
         ex = self.assertRaises(exception.EResourceOperation,
                                profile.do_recover,
@@ -867,7 +954,7 @@ class TestProfileBase(base.SenlinTestCase):
 
         msg = ("Failed in recovering node 'NODE_ID': Failed in creating "
                "STACK: BANNG.")
-        self.assertEqual(msg, six.text_type(ex))
+        self.assertEqual(msg, str(ex))
 
     def test_to_dict(self):
         profile = self._create_profile('test-profile')
